import Paper from "@mui/material/Paper";
import { Fragment, useEffect, useState } from "react";
import Box from "@mui/material/Box";
import { Button, IconButton, Typography } from "@mui/material";
import HelpIcon from "@mui/icons-material/Help";
import CancelIcon from "@mui/icons-material/Cancel";
import axios from "axios";
import { useHistory } from "react-router";
import { Redirect } from "react-router-dom";
import { fetchSigns } from "../helpers/signs";
import { getNextEmail, postAnswer } from "../helpers/gameruns";
import ReactHtmlParser from "react-html-parser";
// TODO repair order
let initVal = {
  subject: "",
  sender: "",
  body: "",
  sign: [],
};

// Play page handles rendering gamerun email  and posting answers
export default function PlayPage() {
  // msg containing if answer was correct
  const [msg, setMsg] = useState("");
  const [currentEmail, setCurrentEmail] = useState(initVal);
  // all signs
  const [signs, setSigns] = useState([]);
  // true if help window is opened, otherwise false
  const [helpWindowOpened, setHelpWindowOpened] = useState(false);
  // answered - if player has answered to current email, answer - player's answer
  // true - phishing email, false - legitimate email
  const [answer, setAnswer] = useState({ answered: false, answer: false });
  const history = useHistory();
  // order of the current email in gamerun
  const [order, setOrder] = useState(0);

  // get next unanswered email
  function handleNextEmail() {
    getNextEmail().then((res) => {
      if (res.data.end) {
        history.push("/summary");
      } else {
        setAnswer({ answered: false, answer: false });
        setCurrentEmail(res.data);
        setMsg("");
        setOrder(order + 1);
      }
    });
    fetchSigns().then((fSigns) => setSigns(fSigns));
  }

  // on page mount fetch next email
  useEffect(() => {
    handleNextEmail();
  }, []);

  // handler for parsing and posting answer
  function handleAnswer(e) {
    // player's answer
    let pAnswer = e.target.value === "true";
    // get type of current email
    let type = currentEmail["type"].type;
    // true if email is phishing (other type than legitimate), otherwise false
    let emailIsPhishing = type !== "legitimate";

    setAnswer({ answered: true, answer: pAnswer });

    // post answer, backend is handling current gamerun and email
    postAnswer(pAnswer);

    //if answer was correct and email is legitimate
    if (pAnswer === false && emailIsPhishing === false) {
      setMsg("Correct! This is a legitimate email");
    }
    // else if and was correct and email is phishing
    else if (pAnswer === true && emailIsPhishing === true) {
      setMsg("Correct! This is a phishing email (" + type + ")");
    }
    // else if answer was incorrect and email is phishing
    else if (pAnswer === false && emailIsPhishing === true) {
      setMsg("Wrong! This is a phishing email (" + type + ")");
    }
    // else if answer was incorrect and email is legitimate
    else {
      setMsg("Wrong! This is a legitimate email");
    }
  }

  // window with email's sender, subject and body
  // contains also answer buttons
  const emailWindow = (
    <Paper sx={{ width: "70%", height: 500 }}>
      <Box
        sx={{
          display: "flex",
          flexDirection: "column",
          width: "95%",
          height: "20%",
          margin: "auto",
          justifyContent: "center",
        }}
      >
        <Typography variant="h5">{currentEmail.subject}</Typography>
        <Typography variant="body2">{currentEmail.sender}</Typography>
      </Box>
      <Box
        sx={{
          width: "95%",
          height: "70%",
          margin: "auto",
        }}
      >
        <Paper
          sx={{
            width: "100%",
            backgroundColor: "#a8a8a8",
            height: "100%",
            overflowY: "auto",
            border: 1,
          }}
        >
          <Typography
            sx={{ wordWrap: "break-word", padding: 2 }}
            variant="body2"
            color="black"
          >
            {ReactHtmlParser(currentEmail.body)}
          </Typography>
        </Paper>
      </Box>
      <Box
        sx={{
          display: "flex",
          width: "100%",
          height: "10%",
          justifyContent: "space-evenly",
          alignItems: "center",
        }}
      >
        {/* Legitimate button */}
        <Button
          variant="contained"
          sx={{ width: "110px" }}
          value={false}
          disabled={answer.answered}
          onClick={handleAnswer}
        >
          Legitimate
        </Button>
        {/* order of email in gamerun */}
        <Typography>{order}</Typography>
        {/* Phishing button */}
        <Button
          variant="contained"
          sx={{ width: "110px" }}
          value={true}
          disabled={answer.answered}
          onClick={handleAnswer}
        >
          Phishing
        </Button>
      </Box>
    </Paper>
  );

  // help window contains all signs,
  const helpWindow = (
    <Paper sx={{ height: 500, width: "35%" }}>
      {/* Button, closes the window */}
      <Box sx={{ display: "flex", justifyContent: "flex-end" }}>
        <IconButton onClick={handleHelpWindow} color="primary">
          <CancelIcon />
        </IconButton>
      </Box>
      <Box sx={{ pl: 2, overflowY: "auto" }}>
        <Typography variant="h6">HINTS</Typography>
      </Box>
      <Box
        sx={{
          height: "70%",
          px: 2,
          mt: 1,
          justifyContent: "center",
          overflowY: "auto",
        }}
      >
        {signs.map((sign, index) => {
          return (
            <Fragment>
              <Typography>
                {index + 1}: {sign.text}
              </Typography>
            </Fragment>
          );
        })}
      </Box>
    </Paper>
  );

  // Indicators windows is shown when email is answered, contains signs of actual email
  function indicatorsWindow() {
    return (
      <Paper sx={{ height: 500, width: "35%" }}>
        <Box sx={{ padding: 2, overflowY: "auto" }}>
          <Typography variant="h6" sx={{ wordWrap: "break-word" }}>
            Indicators
          </Typography>
        </Box>
        <Box
          sx={{ padding: 2, overflowY: "auto", height: "75%", width: "100%" }}
        >
          {currentEmail.sign.map((sign, index) => {
            return (
              <Typography key={index}>
                {index + 1}: {sign.text}
              </Typography>
            );
          })}
        </Box>
        <Box
          sx={{
            display: "flex",
            justifyContent: "flex-end",
            height: "8%",
          }}
        >
          {/* Next button, onlick gets next email of gamerun */}
          <Button
            variant="contained"
            color="primary"
            sx={{ mr: 2 }}
            onClick={handleNextEmail}
            size="medium"
          >
            Next
          </Button>
        </Box>
      </Paper>
    );
  }

  function handleHelpWindow() {
    setHelpWindowOpened(!helpWindowOpened);
  }

  return (
    <Box sx={{ height: "100%", width: "100%" }}>
      {/* Paper with message containing answer correctness and type of email */}
      <Paper
        sx={{
          width: "90%",
          height: "10vh",
          display: "flex",
          alignItems: "center",
          justifyContent: "center",
          margin: "auto",
          mt: 1,
        }}
      >
        <Typography>{msg}</Typography>
      </Paper>
      {/* flexbox with emailWindow, helpWindow and indicatorsWindow */}
      <Box
        sx={{
          display: "flex",
          width: "100%",
          height: "80%",
          alignItems: "center",
          justifyContent: "space-evenly",
        }}
      >
        {emailWindow}
        {
          // if help button was clicked and email is unanswered, open the helpWindow
          helpWindowOpened === true && !answer.answered ? helpWindow : ""
        }
        {
          // if email is answered, show indicatorsWindow containing signs of current email
          answer.answered ? indicatorsWindow() : ""
        }
        {/* Help button */}
        <Box
          sx={{
            display: "flex",
            justifyContent: "flex-end",
            width: "100%",
            position: "absolute",
            pr: 2,
          }}
        >
          {
            // if help window is closed and email is unanswered, show help button
            helpWindowOpened || answer.answered ? (
              ""
            ) : (
              <IconButton onClick={handleHelpWindow}>
                <HelpIcon color="primary" />
              </IconButton>
            )
          }
        </Box>
      </Box>
    </Box>
  );
}
