import React from "react";
import { Redirect, Route } from "react-router-dom";
import { connect } from "react-redux";
import { Typography } from "@mui/material";

// Private route checks if user is logged in
const PrivateRoute = ({ component: Component, auth, ...rest }) => (
  <Route
    {...rest}
    render={(props) => {
      // if authentication is loading, show loading text
      if (auth.isLoading) {
        return <Typography variant="h2">Loading...</Typography>;
      }
      // if user is not logged in, redirect to login page
      else if (!auth.isAuthenticated) {
        return <Redirect to="/login" />;
      }
      // if user is logged in, show requested component
      else {
        return <Component {...props} />;
      }
    }}
  />
);

const mapStateToProps = (state) => ({
  auth: state.auth,
});

export default connect(mapStateToProps)(PrivateRoute);
